package StackTrace::Pretty;
use strict;
use warnings;
use utf8;

our $VERSION = '0.0.1';
$VERSION = eval $VERSION;

1;

__END__

=head1 NAME

StackTrace::Pretty - Modules for stacktrace-pretty

=head1 DESCRIPTION

This module and its submodules are intended for internal use.
